// dibdoc.cpp : implementation of the CDibDoc class
//

#include "stdafx.h"   

#include "albuminc.h"
#include "album.h"
#include <limits.h>

#include "dibdoc.h"  
#include "pcx.h"   
#include "wmf.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDibDoc

IMPLEMENT_DYNCREATE(CDibDoc, CDocument)

BEGIN_MESSAGE_MAP(CDibDoc, CDocument)
	//{{AFX_MSG_MAP(CDibDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#define new DEBUG_NEW

///////////////////// Construction/Destruction       /////////////////////
// CDibDoc construction/destruction

CDibDoc::CDibDoc()
{                   
	SState far* st = (SState far*) AfxGetApp()->m_pMainWnd->SendMessage(UM_GETSTATEINFO);
	m_hDIB = NULL;
	m_palDIB = NULL;
	m_sizeDoc = CSize(1,1);     // dummy value to make CScrollView happy    
	m_palDDB = new CPalette; 
	m_DDB = new CBitmap; 
	m_rcThumb = new CRect;	     
	::SetRect(m_rcThumb,0,0,st->m_thumb,st->m_thumb);
}

CDibDoc::~CDibDoc()
{
	if (m_hDIB != NULL)
		::GlobalFree((HGLOBAL) m_hDIB);
	if (m_DDB != NULL)
		delete m_DDB;
	if (m_palDIB != NULL)
		delete m_palDIB;
	if (m_palDDB != NULL)
		delete m_palDDB; 
	if (m_rcThumb != NULL)
		delete m_rcThumb;
}                                              

///////////////////////  On_Message Handlers  //////////////////////////

// DLL function prototype
extern "C"
HDIB FAR PASCAL _export GIFtoDIB(LPSTR fName);

//pszPathName has info about source of data for document
BOOL CDibDoc::OnOpenDocument(const char* pszPathName)
{                  
	CString fName = pszPathName; 
	HGLOBAL     hC; 
	LPSTR	  		D,C;  
	CBitmap  		Bmp;  
	CDC*				pDC;
	
	fName.MakeUpper();
	DeleteContents();
	BeginWaitCursor();

	if ( (fName.Right(4) == "CLIP") && ::IsClipboardFormatAvailable(CF_DIB) )
		{  
		AfxGetApp()->m_pMainWnd->OpenClipboard();
		hC=::GetClipboardData(CF_DIB);
		m_hDIB=(HDIB)::GlobalAlloc(GHND,::GlobalSize(hC));
		D=(LPSTR)::GlobalLock(m_hDIB);
		C=(LPSTR)::GlobalLock(hC);
		::hmemcpy(D,C,::GlobalSize(m_hDIB));
		::GlobalUnlock(m_hDIB);
		::GlobalUnlock(hC);
		::CloseClipboard();
		}  
	else if ( (fName.Right(4) == "CLIP") && ::IsClipboardFormatAvailable(CF_METAFILEPICT) )
		{  
		AfxGetApp()->m_pMainWnd->OpenClipboard();
		hC=::GetClipboardData(CF_METAFILEPICT);
		pDC =AfxGetApp()->m_pMainWnd->GetDC();
		PICTtoDDB(hC,pDC,AfxGetApp()->m_pMainWnd,CRect(0,0,0,0),&Bmp);
		m_hDIB=(HDIB)DDBtoDIB(&Bmp,NULL,pDC); 
		AfxGetApp()->m_pMainWnd->ReleaseDC(pDC);
		::CloseClipboard();
		}
	else if (fName.Right(4) == ".WMF")
		{         
		pDC =AfxGetApp()->m_pMainWnd->GetDC(); 
		hC=LoadAldusFile((LPSTR)pszPathName);
		PICTtoDDB(hC,pDC,AfxGetApp()->m_pMainWnd,CRect(0,0,0,0),&Bmp);
		m_hDIB=(HDIB)DDBtoDIB(&Bmp,NULL,pDC); 
		AfxGetApp()->m_pMainWnd->ReleaseDC(pDC);
		DelPICT(hC);
		}
	else if (fName.Right(4) == ".BMP")
		m_hDIB = ::LoadDIBFile((PSTR)pszPathName, 0);  
	else if (fName.Right(4) == ".PCX")
		m_hDIB = ::LoadPCXFile((PSTR)pszPathName, 0);
	else if (fName.Right(4) == ".GIF") 
		m_hDIB = GIFtoDIB((LPSTR)pszPathName);

	InitDIBData();
	EndWaitCursor();

	if (m_hDIB == NULL)
	{
		// may not be DIB format
		MessageBox(NULL, "Couldn't load Image", NULL,
						 MB_ICONINFORMATION | MB_OK);
		return FALSE;
	}
	SetPathName(pszPathName);
	SetModifiedFlag(FALSE);     // start off with unmodified
	return TRUE;
}

/////////////////////////////  Operations    /////////////////////
// CDibDoc 

CSize CDibDoc::GetDocSize()
{
 		return m_sizeDoc;
}

void CDibDoc::InitDIBData()
{
	if (m_palDIB != NULL)
		{
		delete m_palDIB;
		m_palDIB = NULL;
		}
	if (m_hDIB == NULL)
		return;

	// Set up document size
	LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) m_hDIB);
	if (::DIBWidth(lpDIB) > INT_MAX ||::DIBHeight(lpDIB) > INT_MAX)
		{
		::GlobalUnlock((HGLOBAL) m_hDIB);
		::GlobalFree((HGLOBAL) m_hDIB);
		m_hDIB = NULL;
		MessageBox(NULL, "DIB is too large", NULL,
						 MB_ICONINFORMATION | MB_OK);
		return;
		}
	m_sizeDoc = CSize((int) ::DIBWidth(lpDIB), (int) ::DIBHeight(lpDIB));
	::GlobalUnlock((HGLOBAL) m_hDIB);
	// Create copy of palette
	m_palDIB = new CPalette;
	if (m_palDIB == NULL)
		{
		// we must be really low on memory
		::GlobalFree((HGLOBAL) m_hDIB);
		m_hDIB = NULL;
		return;
		}
	if (::CreateDIBPalette(m_hDIB, m_palDIB) == NULL)  // DIB may not have a palette
		m_palDIB->CreateStockObject(DEFAULT_PALETTE);
	if (m_hDIB != NULL)  
		InitDDBData();		
}

void CDibDoc::InitDDBData()
{
	DIBtoDDB((HGLOBAL)m_hDIB,m_DDB,AfxGetApp()->m_pMainWnd,m_palDDB,*m_rcThumb);
  AfxCheckMemory();
}  

void CDibDoc::ReplaceHDIB(HDIB hDIB)
{
	if (m_hDIB != NULL)
		{
		::GlobalFree((HGLOBAL) m_hDIB);
		}
	m_hDIB = hDIB;
}

/////////////////////////////////////////////////////////////////////////////
// CDibDoc diagnostics

#ifdef _DEBUG
void CDibDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDibDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}

#endif //_DEBUG

