// mainfrm.cpp : implementation of the CMainFrame class
// dovermyer

#include "stdafx.h"
#include "albuminc.h" 

#include "album.h"
#include "mainfrm.h" 
#include "dibdoc.h"
#include "dibview.h"      
#include "config.h"  
#include <ctl3d.h>

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_COMMAND(ID_WINDOW_CLOSEALL, OnWindowCloseall)
	ON_COMMAND(ID_EDIT_CLEAR_CLIP, OnEditClearClip)
	ON_COMMAND(IDD_CONFIGURE, OnConfigure) 
	ON_WM_CLOSE()
	ON_WM_CREATE()
	ON_WM_PALETTECHANGED()
	ON_WM_QUERYNEWPALETTE()   
	ON_WM_SYSCOLORCHANGE()
	ON_WM_TIMER()
	ON_MESSAGE(UM_GETSTATEINFO, OnGetStateInfo)     
	ON_MESSAGE(UM_GETSTATUSBAR, OnGetStatusBar)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR_CLIP, OnUpdateEditClearClip)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_CLIPBOARD, OnUpdateIndicatorClipboard) 
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// arrays of IDs used to initialize control bars

// toolbar buttons - IDs are command buttons
static UINT BASED_CODE buttons[] =
{
	// same order as in the bitmap 'toolbar.bmp'
	ID_FILE_OPEN,
		ID_SEPARATOR,
	ID_EDIT_COPY,
	ID_EDIT_PASTE, 
	ID_EDIT_CLEAR_CLIP,
		ID_SEPARATOR,
	ID_WINDOW_TILE_HORZ,
	ID_FILE_PRINT,
	ID_HELP_INDEX,
};

static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CLIPBOARD,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

//////////////////////  Birth & Death      ///////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{                                           
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadBitmap(IDR_MAINFRAME) ||
		!m_wndToolBar.SetButtons(buttons,
		  sizeof(buttons)/sizeof(UINT)))
		{
		TRACE("Failed to create toolbar\n");
		return -1;      // fail to create
		}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
		{
		TRACE("Failed to create status bar\n");
		return -1;      // fail to create
		}   
	int nIndex = 0, cxWidth;
	UINT nID, nStyle;
	m_wndStatusBar.GetPaneInfo(nIndex,nID, nStyle,cxWidth);
	m_wndStatusBar.SetPaneInfo(nIndex,nID,SBPS_STRETCH | SBPS_NORMAL,cxWidth);
	return 0;
}


////////////////////////////    On Methods     ///////////////////////
// CMainFrame commands  

void CMainFrame::OnClose()
{
	CMDIFrameWnd::OnClose();
} 

void CMainFrame::OnConfigure()
{ 
	SState  st = GetApp()->GetStateInfo();
	CConfig dlg(this); 
	
	dlg.m_lMargin= st.m_lMargin; 
	dlg.m_tMargin= st.m_tMargin;
	dlg.m_rMargin= st.m_rMargin;
	dlg.m_bMargin= st.m_bMargin;
	dlg.m_thumb  = st.m_thumb;
	dlg.m_across = st.m_across;
	dlg.m_down   = st.m_down;
	dlg.DoModal();
	st.m_lMargin= dlg.m_lMargin; 
	st.m_tMargin= dlg.m_tMargin;
	st.m_rMargin= dlg.m_rMargin;
	st.m_bMargin= dlg.m_bMargin;
	st.m_thumb  = dlg.m_thumb;
	st.m_across = dlg.m_across;
	st.m_down   = dlg.m_down;   
	GetApp()->SetStateInfo(st);
}                   

void CMainFrame::OnEditClearClip()
{
	if (OpenClipboard())
		{
		EmptyClipboard();
		CloseClipboard();
		}
}   

LRESULT CMainFrame::OnGetStateInfo(WPARAM wParam, LPARAM lParam)
{                                                               
	SState* pState = &(GetApp()->m_State);
	return (LONG)(SState far*)pState;  
}   

void CMainFrame::OnSysColorChange()
{
	Ctl3dColorChange();	
}
      
LRESULT CMainFrame::OnGetStatusBar(WPARAM wParam, LPARAM lParam) 
{
	return (LRESULT) MAKELRESULT(&m_wndStatusBar,0);
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd)
{
	CMDIFrameWnd::OnPaletteChanged(pFocusWnd);   
	SState st = GetApp()->GetStateInfo();
	
	if (st.bIsNoUpdate == TRUE)
		return;
  if ( IsIconic() )
  	return;  // skip it if iconic
	// always realize the palette for the active view
	CMDIChildWnd* pMDIChildWnd = MDIGetActive();
	if (pMDIChildWnd == NULL)
		return; // no active MDI child frame
	CView* pView = pMDIChildWnd->GetActiveView();
	ASSERT(pView != NULL);
	// notify all child windows that the palette has changed
	SendMessageToDescendants(WM_DOREALIZE, (WPARAM)pView->m_hWnd);
}

BOOL CMainFrame::OnQueryNewPalette()
{                                      
	SState st = GetApp()->GetStateInfo();	
	if (st.bIsNoUpdate == TRUE)
		return FALSE;

	if ( (IsIconic()) )
		return FALSE; // skip it if iconic
	// always realize the palette for the active view
	CMDIChildWnd* pMDIChildWnd = MDIGetActive();
	if (pMDIChildWnd == NULL)
		return FALSE; // no active MDI child frame (no new palette)
	CView* pView = pMDIChildWnd->GetActiveView();
	ASSERT(pView != NULL);

	// just notify the target view
	pView->SendMessage(WM_DOREALIZE, (WPARAM)pView->m_hWnd);
	return TRUE;
}

void CMainFrame::OnTimer(UINT nIDEvent)
{
	CRect  r(GetApp()->m_rcSplash);
	r.OffsetRect(100,100);
	::InvalidateRect(0,&r,TRUE);
	
	ShowWindow(SW_NORMAL);    
	KillTimer(1);
	CMDIFrameWnd::OnTimer(nIDEvent);
}

void CMainFrame::OnWindowCloseall()
{                
	POSITION pos;
	CDocument* pDoc;
	CObList* pDocList = GetApp()->GetDocList();

	pos=pDocList->GetHeadPosition();
	while (pos != NULL)
		{
		pDoc=(CDocument*)pDocList->GetNext(pos);
		pDoc->OnCloseDocument();
		}
}

/////////////////////////    OnUpdate  Methods ///////////////////////////
void CMainFrame::OnUpdateEditClearClip(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(::CountClipboardFormats());
	
} 

void CMainFrame::OnUpdateIndicatorClipboard(CCmdUI* pCmdUI)
{
	GetApp()->OnUpdateEditPaste(pCmdUI);	
}


