// prntdoc.cpp : implementation of the CPrntDoc class 
// supports printing a list of graphics files - bmp,pcx,gif and wmf formats  
//    are converted into an hDIB and hPalette
// files are read and printed one at a time, reducing resource requirements
// DOvermyer 7/1/93


#include "stdafx.h"
#include "album.h"
#include <limits.h>
                  
#include "wmf.h"                  
#include "pcx.h"      
#include "mainfrm.h"
#include "prntdoc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPrntDoc

IMPLEMENT_DYNCREATE(CPrntDoc, CDocument)

BEGIN_MESSAGE_MAP(CPrntDoc, CDocument)
	//{{AFX_MSG_MAP(CPrntDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#define new DEBUG_NEW
/////////////////////////  Construction/Destruction  ///////////////////////////////
// CPrntDoc construction/destruction

CPrntDoc::CPrntDoc()
{
	m_hDIB = NULL;
	m_palDIB = NULL;
	m_sizeDoc = CSize(1,1);     
	m_rcThumb = new CRect;     
	::SetRect(m_rcThumb,0,0,100,100);
}

CPrntDoc::~CPrntDoc()
{
	if (m_hDIB != NULL)
		::GlobalFree((HGLOBAL) m_hDIB);
	if (m_palDIB != NULL)
		delete m_palDIB;
	if (m_rcThumb != NULL)
		delete m_rcThumb;            
} 
///////////////////  Operations  ////////////////////////////////////////////////////


void CPrntDoc::InitDIBData()
{
	if (m_palDIB != NULL)
		{
		delete m_palDIB;
		m_palDIB = NULL;
		}
	if (m_hDIB == NULL)
		{
		return;
		}
	// Set up document size
	LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) m_hDIB);
	if (::DIBWidth(lpDIB) > INT_MAX ||::DIBHeight(lpDIB) > INT_MAX)
		{
		::GlobalUnlock((HGLOBAL) m_hDIB);
		::GlobalFree((HGLOBAL) m_hDIB);
		m_hDIB = NULL;
		MessageBox(NULL, "DIB is too large", NULL,
						 MB_ICONINFORMATION | MB_OK);
		return;
		}
	m_sizeDoc = CSize((int) ::DIBWidth(lpDIB), (int) ::DIBHeight(lpDIB));
	::GlobalUnlock((HGLOBAL) m_hDIB);
	// Create a palette based on DIB color table
	m_palDIB = new CPalette;
	if (m_palDIB == NULL)
		{
		// we must be really low on memory
		::GlobalFree((HGLOBAL) m_hDIB);
		m_hDIB = NULL;
		return;
		}
	if (::CreateDIBPalette(m_hDIB, m_palDIB) == NULL)  // DIB may not have a palette
		m_palDIB->CreateStockObject(DEFAULT_PALETTE);
}

extern "C"     // I couldn't bear to rewrite this from pascal...
HDIB FAR PASCAL _export GIFToDIB(LPSTR fName);
                                                    
//build an hDIB from the pathname in dibName                                                    
void CPrntDoc::InitDIB(const char* dibName)
{     
	HGLOBAL		hC = 0;
	CBitmap		Bmp;
	CDC*			pDC;
	if (m_hDIB != NULL)
		{
		::GlobalFree((HGLOBAL) m_hDIB);
		}  
	if (dibName != NULL)
		{
		
		CString fName = dibName;  //make a copy
		fName.MakeUpper();        //upper case
		BeginWaitCursor();
                        
    if (fName.Right(4) == ".WMF")  //it had better be an Aldus placeable metafile!
    	{
    	pDC=AfxGetApp()->m_pMainWnd->GetDC();
    	hC=LoadAldusFile((LPSTR)dibName);
    	PICTtoDDB(hC,pDC,AfxGetApp()->m_pMainWnd,CRect(0,0,0,0),&Bmp);
    	m_hDIB=(HDIB)DDBtoDIB(&Bmp,NULL,pDC);
    	AfxGetApp()->m_pMainWnd->ReleaseDC(pDC);
    	DelPICT(hC);
    	}                    
		if (fName.Right(4) == ".BMP")   //DIBs only please
			m_hDIB = ::LoadDIBFile((PSTR)dibName, 0);  
		else if (fName.Right(4) == ".PCX")     // version 5  2,4,8 & 24 bit images
			m_hDIB = ::LoadPCXFile((PSTR)dibName, 0);
		else if (fName.Right(4) == ".GIF")     // 2,4,8 bit single image , interlaced & non-interlaced
			m_hDIB = GIFToDIB((LPSTR)dibName);

		InitDIBData();
		EndWaitCursor();
		}
}

////////////////////////////  On Handlers  ////////////////////////////////
BOOL CPrntDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

//pszPathName = global buffer returned by menu item "File/Print List" common dialog box 
//reformat and save as list of pathnames
BOOL CPrntDoc::OnOpenDocument(const char* pszPathName)
{   
	char* pbuf;  
	CString path,pathname;
	  
	pbuf = new char [strlen(pszPathName)+1];
	strcpy(pbuf,pszPathName);  
	char* pbuf1 = pbuf;	
	
	pbuf1=strtok(pbuf," ");
	path=pbuf1;  
	pbuf1=strtok(NULL," ");   
	if (pbuf1 == NULL)
			m_fileList.AddTail(pbuf);
	else
		{	  
		if (path[path.GetLength()-1]=='\\') 
				path = path.Left(path.GetLength()-1);
  	while (pbuf1 != NULL)
				{                  
				pathname=path + '\\' + pbuf1;
				m_fileList.AddTail(pathname);
				pbuf1=strtok(NULL," ");
				} 
	  }
						 
	delete [] pbuf;    
	SState far* st = (SState far*) AfxGetApp()->m_pMainWnd->SendMessage(UM_GETSTATEINFO); 
	m_State = *st; // grab a copy of the state info
	return TRUE;
}
/////////////////////////////////////////////////////////////////////////////
// CPrntDoc diagnostics

#ifdef _DEBUG
void CPrntDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CPrntDoc::Dump(CDumpContext& dc) const
{                           
	char nl[2]="\n";
	
	CDocument::Dump(dc); 
	dc << "m_hDIB" << m_hDIB << nl;
	dc << "m_palDIB" << m_palDIB <<nl;
	dc << "m_sizeDoc" << m_sizeDoc << nl;
	dc << "m_rcThumb" << *m_rcThumb << nl;
	dc << "m_fileList" << m_fileList << nl;
}

#endif //_DEBUG

